#include "base64.h"

/*------------------------------------------------------------------------------
 Encode un string. Retourne un nouveau string qu'il faudra librer
------------------------------------------------------------------------------*/
char* base64_encoder(char *input)
{
 unsigned char in[3], out[4];
 int i, len, p1=0, p2=0, size = strlen(input)+1;
 unsigned char output[size*4/3 + 1];

 while (p1 < size)
 {
  len = 0;
  for (i=0; i<3; i++)
  {
   in[i] = (unsigned char) input[p1++];
   if (p1 < size) len++; else in[i] = 0;
  }
  if (len)
  {
   encode_b64_block(in, out, len);
   for (i=0; i<4; i++)
    output[p2++] = out[i];
  }
 }

 output[p2]=0;
 return strdup(output);
}


void encode_b64_block( unsigned char in[3], unsigned char out[4], int len )
{
 out[0] = cb64[in[0] >> 2];
 out[1] = cb64[((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4)];
 out[2] = (unsigned char) (len > 1 ? cb64[ ((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6)] : '=');
 out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f] : '=');
}

/*------------------------------------------------------------------------------
  ENCODE UN FICHIER EN BASE64
------------------------------------------------------------------------------*/
unsigned char *encode_b64_file(char *fic, long *taille)
{
  FILE *pF = fopen(fic, "rb");
  if (!pF) return;
  fseek(pF, 0, SEEK_END);
  //long size = ftell(pF);

  *taille = ftell(pF);
  *taille *= 4;
  *taille /= 3;
  *taille++;
  rewind(pF);

  //cre le string final
  unsigned char *s = malloc(*taille);//size*(4/3)+1);

  unsigned char in[3], out[4];
  int i, len, start=0;

  while (!feof(pF))
  {
    len = 0;
    for (i=0; i<3; i++)
	{
      in[i] = (unsigned char) getc(pF);
      if (!feof(pF)) len++; else in[i] = 0;
    }
    if (len)
	{
      encode_b64_block(in, s+start, len);
      start += 4;
    }
  }
  fclose(pF);

  s[*taille]=0;
  return s;
}

/*------------------------------------------------------------------------------
  Wordwrap d'un string. Retourne un string qu'il faudra librer.
      s         : le string  wrapper
      taille    : sa taille en octets
      dsttaille : taille du string final, modifi par la fonction
      width     : le string final aura width chars par ligne
------------------------------------------------------------------------------*/
char *bxwrap_to_char(char *s, long taille, long *dsttaille, int width)
{
 unsigned int b=0;
 unsigned long n=0, i=0;
 //le char de destination se verra ajouter combien de \n ?
 //int nb =            (int)(taille/width);
 *dsttaille = taille + (int)(taille/width);
 char *t = malloc(*dsttaille);

 while (n < taille)
 {
  if (b<width)
  {
   t[i+b]=s[n++];
   b++;
  }
  else
  {
   i += b;
   t[i++]='\n';
   b=0;
  }
 }

 t[*dsttaille]=0;
 return t;
}



/*------------------------------------------------------------------------------
 Dcoder un string. Retourne un nouveau string qu'il faudra librer
------------------------------------------------------------------------------*/
char* base64_decoder(char *input)
{
 unsigned char in[3], out[4], v;
 int i, len;
 int p1 = 0, p2 = 0, size=strlen(input)+1;

 unsigned char output[ size*3/4 + 1 ];

 while (p1 < size)
 {
  for (len=0, i=0; i< 4 && p1 < size; i++)
  {
   v = 0;
   while (p1<size && v == 0)
   {
    v = (unsigned char) input[p1++];
    v = (unsigned char) ((v < 43 || v > 122) ? 0 : cd64[v - 43]);
    if (v) v = (unsigned char) ((v == '$') ? 0 : v - 61);
   }
   if (p1 < size)
   {
    len++;
    if (v) in[i] = (unsigned char) (v - 1);
   }
   else in[i] = 0;
  }
  if (len)
  {
   decode_b64_block(in, out);
   for (i=0; i<len-1; i++)
    output[p2++] = out[i];
  }
 }

 output[p2] = 0;
 return strdup(output);
}

void decode_b64_block( unsigned char in[4], unsigned char out[3])
{
 out[0] = (unsigned char) (in[0] << 2 | in[1] >> 4);
 out[1] = (unsigned char) (in[1] << 4 | in[2] >> 2);
 out[2] = (unsigned char) (((in[2] << 6) & 0xc0) | in[3]);
}


